function Telecommande() {
    this.client = null;
}

Telecommande.prototype = {
    onLoad: function(settings) {
        // Hostname can be set on url (for debugging purpose)
        var user = "user";
        var password = "password";
        var hostname = new URL(window.location.href).searchParams.get('hostname') || location.hostname;
        this.client = IDAL.getClient(hostname, user, password, { timeout: 2500 });

        var container = document.getElementById("scenario-container");
        for (var i = 0 ; i < settings.length ; i++) {
            var button = document.createElement("button");
            button.type = "button";
            button.className = "btn btn-primary";
            button.innerText = settings[i].title;
            button.addEventListener("click", this.onButtonClick.bind(this, settings[i].actions));

            container.appendChild(button);
        }

        $('#error-modal').modal({});

        IDAL.getRemoteControlInterfaces(hostname, user, password, { timeout: 2500 })
            .then((ifaces) => {
                this.ifaces = ifaces;
                this.initTimeBar();
            });

        var logo = document.getElementById("logo");
        logo.addEventListener("dblclick", () => { this.requestFullScreen(); });
    },

    initTimeBar: function() {
        this.timebar = {
            play: document.getElementById("tb-play"),
            position: document.getElementById("tb-position"),
            duration: document.getElementById("tb-duration"),
            range: document.getElementById("tb-range"),
            bw: document.getElementById("tb-bw"),
            fw: document.getElementById("tb-fw"),
            mute: document.getElementById("tb-mute")
        };

        var onTimebarError = () => this.onError("Erreur lors de l'envoi des commandes.");

        this.timebar.play.addEventListener("click", () => { this.ifaces.player.playPause().catch(onTimebarError); });
        this.timebar.bw.addEventListener("click", () => { this.ifaces.player.seek(-10000,  true).catch(onTimebarError); });
        this.timebar.fw.addEventListener("click", () => { this.ifaces.player.seek(10000,  true).catch(onTimebarError); });
        this.timebar.mute.addEventListener("click", () => { this.ifaces.player.toggleMute().catch(onTimebarError); });
        this.timebar.range.addEventListener("change", (event) => {
            this.ifaces.player.seek(parseInt(this.timebar.range.value) * 1000, false).catch(onTimebarError);
        });

        /**
         * Disable time change from the player events during the time the
         * input range is being modified...
         */
        var allowTimeChange = true;

        this.timebar.range.addEventListener("mousedown", (event) => {
            allowTimeChange = false;
        });

        this.timebar.range.addEventListener("mouseup", (event) => {
            allowTimeChange = true;
        });

        this.timebar.range.addEventListener("touchstart", (event) => {
            allowTimeChange = false;
        });

        this.timebar.range.addEventListener("touchend", (event) => {
            allowTimeChange = true;
        });

        var setTimebarDuration = (duration) => {
            if (allowTimeChange) {
                this.timebar.duration.innerText = this.getTimeString(duration);
                this.timebar.range.max = Math.trunc(duration);
            }
        };

        var setTimebarPosition = (position) => {
            if (allowTimeChange) {
                this.timebar.position.innerText = this.getTimeString(position);
                this.timebar.range.value = Math.trunc(position);
            }
        };

        var setTimebarMuteState = (mute) => {
            var classList = this.timebar.mute.classList;
            if (mute) {
                classList.remove("icon-volume-off");
                classList.add("icon-volume-on");
            } else {
                classList.add("icon-volume-off");
                classList.remove("icon-volume-on");
            }
        };

        var setTimebarPlayState = (state) => {
            var classList = this.timebar.play.classList;
            if (state == 'playing') {
                classList.add("icon-pause");
                classList.remove("icon-play");
            } else {
                classList.remove("icon-pause");
                classList.add("icon-play");
            }
        };

        this.ifaces.player.getMute().then(setTimebarMuteState);
        this.ifaces.player.getState().then(setTimebarPlayState);
        this.ifaces.player.getPosition().then(setTimebarPosition);
        this.ifaces.player.getDuration().then(setTimebarDuration);

        this.ifaces.player.addListener((e) => {
            if (e.detail.type == 'time-update') {
                setTimebarPosition(e.detail.position);
                setTimebarDuration(e.detail.duration);
            } else if (e.detail.type == 'mute-change') {
                setTimebarMuteState(e.detail.mute);
            } else if (e.detail.type == "state-change") {
                setTimebarPlayState(e.detail.state);
            }
        });
    },


    getTimeString: function(d) {
        var remain = Math.trunc(d);
        var seconds = remain % 60;
        remain -= seconds;
        var minutes = remain / 60;

        return "" + (minutes < 10 ? '0' + minutes : minutes) + ":" + (seconds < 10 ? '0' + seconds : seconds);
    },

    onButtonClick: function(actions) {
        var all = actions.slice();
        var client = this.client;

        function foreachAction() {
            var f = all.shift();
            if (f) {
                return client.post(f)
                    .then(() => {
                        return foreachAction();
                    });
            }
            return Promise.resolve();
        }

        foreachAction()
            .catch(() => this.onError("Erreur lors de l'envoi des commandes."))
    },

    onError: function(message) {
        var target = document.getElementById("error-message");
        target.innerText = message;
        $('#error-modal').modal('show');
    },

    requestFullScreen: function() {
        var doc = window.document;
        if (!doc.fullscreenElement && !doc.mozFullScreenElement && !doc.webkitFullscreenElement && !doc.msFullscreenElement) {
            const docEl = doc.documentElement;
            const requestFullScreen = docEl.requestFullscreen ||
                docEl.mozRequestFullScreen ||
                docEl.webkitRequestFullScreen ||
                docEl.msRequestFullscreen;
            if (requestFullScreen) {
                // Fullscreen request can only be requested by user gesture, so may
                // throw an exception...
                try {
                    requestFullScreen.call(docEl);
                } catch(e) {

                }
            }
        }
    }
}

var g_app = new Telecommande();

window.addEventListener("load", () => g_app.onLoad([
    { title: "Home page", actions: [ "/api/player/play-folder?folder=1" ] },
    { title: "Video player", actions: [ "/api/player/play-folder?folder=2" ] },
    { title: "Show control", actions: [ "/api/player/play-folder?folder=4" ] },
    { title: "Vision", actions: [ "/api/player/play-folder?folder=8" ] },
    { title: "Case studies", actions: [ "/api/player/play-folder?folder=16" ] },
]));